import re
import csv
import grok
import cPickle
from cStringIO import StringIO
from xml.dom.minidom import Document, getDOMImplementation
from waeup.interfaces import IWAeUPObject, IWAeUPExporter, IWAeUPXMLExporter

def readFile(f):
    """read a CSV file"""
    headers = []
    rows = []
    f = csv.reader(f)
    headers = f.next()
    for line in f:
        rows.append(line)
    return (headers, rows)

def writeFile(data):
    writer = csv.writer(open("export.csv", "wb"))
    writer.writerows(data)

class Exporter(grok.Adapter):
    """Export a WAeUP object.

    This all-purpose exporter exports attributes defined in schemata
    and contained objects (if the exported thing is a container).
    """
    grok.context(IWAeUPObject)
    grok.provides(IWAeUPExporter)

    def export(self, obj, filepath=None):
        if filepath is None:
            filelike_obj = StringIO()
        else:
            filelike_obj = open(filepath, 'wb')
        exported_obj = cPickle.dump(obj, filelike_obj)
        filelike_obj.close()
        return filelike_obj



ts_regexp = re.compile('^<type (.*)>$')
def getTypeString(obj):
    type_str = str(type(obj))
    m = ts_regexp.match(type_str).groups()
    return m[0][1:-1]
    


class XMLExporter(grok.Adapter):
    """Export a WAeUP object as XML.

    This all-purpose exporter exports attributes defined in schemata
    and contained objects (if the exported thing is a container).
    """
    grok.context(IWAeUPObject)
    grok.provides(IWAeUPXMLExporter)

    doc = None
    
    def getObjectNode(self, obj):
        dotted_name = obj.__module__ + '.' + obj.__class__.__name__
        node = self.doc.createElement('object')
        node.setAttribute('type', dotted_name)
        return node
    
    def _export(self, obj, parent=None):
        if self.doc is None:
            impl = getDOMImplementation()
            self.doc = impl.createDocument(None, 'waeupdata', None)
        if parent is None:
            parent = self.doc.documentElement
        
        # Handle object meta data...
        obj_node = self.getObjectNode(obj)
        parent.appendChild(obj_node)
        parent = obj_node

        # Handle attributes...
        for name in obj.__dict__:
            if name.startswith('_'):
                continue
            val = unicode(getattr(obj, 'name'))
            type_name = getTypeString(getattr(obj, 'name'))
            newnode = self.doc.createElement('attribute')
            newnode.setAttribute('name', name)
            newnode.setAttribute('type', type_name)
            text = self.doc.createTextNode(val)
            newnode.appendChild(text)
            parent.appendChild(newnode)
        return self.doc

    def cleanUp(self):
        if self.doc is not None:
            self.doc.unlink()
        self.doc = None
        return

    def export(self, obj, filepath=None):
        result = self._export(obj)
        if filepath is None:
            filelike_obj = StringIO()
        else:
            filelike_obj = open(filepath, 'wb')
        filelike_obj.write(result.toprettyxml(encoding='UTF-8'))
        filelike_obj.seek(0)
        self.cleanUp()
        return filelike_obj

    def exportPrettyXML(self, obj):
        result = self._export(obj).toprettyxml(encoding='UTF-8')
        self.cleanUp()
        return result
